## plot methods
setMethod("plot", signature(x="cosmo"), function(x, y="missing", type="PWM",...) {
  if (type == "PWM"){
    seqLogo(x@pwm)

  }else if (type == "prob"){

    seqs <- x@seqs
    numSeqs <- length(seqs)

    seqList <- list()
    seqNames <- list()
    seqLengths <- NULL
    for (i in 1:length(seqs)){
      seqList[[i]] <- seqs[[i]]$seq
      seqNames[[i]] <- seqs[[i]]$desc
      seqLengths[i] <- nchar(seqs[[i]]$seq)
    }
    maxLen <- max(seqLengths)
    
    ## now convert to matrix
    seqMat <- seqList2seqMat(seqList)
    probs <- x@probs

    colors <-seqMat
    colors[colors==1]<-"green"
    colors[colors==2]<-"blue"
    colors[colors==3]<-"orange"
    colors[colors==4]<-"red"
    colors[colors==5]<-"grey"
    

    ht <- (numSeqs/50)*17 + 3;
    wt<-18;
    fac <- (ht/18)**0.55;
    
    par(oma<-c(0,8,0,0))
    plot(1:maxLen,1:maxLen,ylim=c(0,numSeqs+1),xlim=c(1,maxLen),type="n",axes=F,xlab="",ylab="")
    axis(side=2,at=seq(1,numSeqs,by=1), labels = rev(unlist(seqNames)),las=2,lty=0,cex.axis=0.75/fac)
    axis(side=1,cex.axis=0.75/fac)
    for (i in 1:numSeqs){
      len = seqLengths[i]    
      segments(1:len,rep(numSeqs+1-i,len),1:len,numSeqs+1-i+0.5*probs[[i]],col=colors[1:len,i],lwd=2.5)
    }
        
  }else{
    stop("type must be either 'PWM' or 'prob'")
  }
  
})




setMethod("plot", signature(x="constraintSet"),
          function(x, y="missing", varLen=4, propLen=4, plot.IC=TRUE, plot.nucFreq=TRUE,...){

    par(mfrow=c(1, 1), mar=c(5, 3, 4, 4.5) + 0.1, las=1)
    int=unlist(lapply(x@intervals, function(y){
        if (y@type == "BasePairs" | y@type == "B" | y@type == "b") {
            y@length
        }
        else if (y@type == "Variable" | y@type == "V" | y@type == "v") {
            varLen
        }
        else if (y@type == "Proportion" | y@type == "P" | y@type == "p") {
            propLen
        }}))    
    props <- unlist(lapply(x@intervals, function(y){y@prop}))
    types <- unlist(lapply(x@intervals, function(y){
        y@type
    }))
    labels <- c()
    for (i in 1:length(types)) {
        if (types[i] == "BasePairs" |
            types[i] == "B" | types[i] == "b") {
          labels <- c(labels, 1:int[i])}
        if (types[i] == "Proportion" |
            types[i] == "P" | types[i] == "p") {
          labels <- c(labels, paste(props[i], "%", sep=""))}
        if (types[i] == "Variable" |
            types[i] == "V" | types[i] == "v") {
          labels <- c(labels, "?")}
    }
    splits <- c(0, cumsum(int))
    lwbd <- min(0.49 + 0.032*sum(int), 1)
    text <- ifelse(nrow(x@subMotifCon) == 0, "Constraint Plot",
                   paste("Constraint Plot \n Submotif present:",
                         as.character(x@subMotifCon[, "Submotif"])))
    plot(0, xlab="", ylab="", main=text,
         xlim=c(lwbd, sum(int) + 1-lwbd),
         ylim=c(0.07, 1.93), col=0, axes=FALSE)
    axis(side=2, at=c(0, 1, 2))
    axis(side=4, at=c(0, 0.5, 1, 1.5, 2),
         labels=c("0%", "25%", "50%", "75%", "100%"))
    par(las=0)
    mtext(text="Information Content", side=2, line=1.9)
    mtext(text="Proportion", side=4, line=2.7)
    abline(v=splits[-c(1, length(splits))] + 0.5, lty=2)
    abline(h=c(0, 2))
    rug(1:(sum(int) + 1)-0.5, ticksize=-0.02, quiet=TRUE)
    if (length(splits[-c(1, length(splits))])!=0) {
      rug(splits[-c(1, length(splits))] + 1/2, ticksize=-0.12, lty=2)}
    ats <- c()
    counting <- 0
    for (i in 1:length(types)) {
        if (types[i] == "BasePairs" |
            types[i] == "B" | types[i] == "b") {
          ats <- c(ats, counting + 1:int[i])
          counting <- counting + int[i]}
        if (types[i] == "Proportion" |
            types[i] == "P" | types[i] == "p") {
          ats <- c(ats, counting + (propLen + 1)/2)
          counting <- counting + propLen}
        if (types[i] == "Variable" |
            types[i] == "V" | types[i] == "v") {
          ats <- c(ats, counting + (varLen + 1)/2)
          counting <- counting + varLen}
    }
    mtext(labels , side=1 , at=ats, line=0.5)
    mtext(types, at=(splits[-1] + splits[-length(splits)])/2 + 1/2, side=1, line=2)
    
    for (i in 1:x@numIntervals){

        if (plot.IC) {

            if(dim(x@intervals[[i]]@boundedCon)[1]!=0) {
                lines(c(splits[i], splits[i + 1]) + 1/2,
                      rep(x@intervals[[i]]@boundedCon$Lower, 2), lwd=3, col="red")
                lines(c(splits[i], splits[i + 1]) + 1/2,
                      rep(x@intervals[[i]]@boundedCon$Upper, 2), lwd=3, col="red")
            }

            if (dim(x@intervals[[i]]@shapeCon)[1]!=0){
                lines(c(splits[i], splits[i + 1]) + 1/2,
                      c(x@intervals[[i]]@shapeCon$StartLower,
                        x@intervals[[i]]@shapeCon$EndLower), lwd=3, col="red")
                lines(c(splits[i], splits[i + 1]) + 1/2,
                      c(x@intervals[[i]]@shapeCon$StartUpper,
                        x@intervals[[i]]@shapeCon$EndUpper), lwd=3, col="red")
            }

        }

        if (plot.nucFreq){

            if (dim(x@intervals[[i]]@posFreqCon)[1]!=0){
                startlevel=0
                if (sum(x@intervals[[i]]@posFreqCon[, 2] == "All")>0){
                    dint <- x@intervals[[i]]@posFreqCon[x@intervals[[i]]@posFreqCon[, 2] == "All",]
                    for (k in 1:nrow(dint)) {
                        rect(splits[i] + 1/2, startlevel,
                             splits[i + 1] + 1/2,
                             startlevel + 2*dint$Lower[k],
                             density=20, angle=45, col="blue")
                        l <- sum(int)
                        legend((splits[i] + splits[i + 1])/2 +
                               0.4748-0.0659*l, startlevel +
                               dint$Lower[k] + 0.1,
                               legend=dint$Nucleotide[k], bty="n")
                        startlevel <- startlevel + 2*dint$Lower[k]
                    }
                }
                if (sum(x@intervals[[i]]@posFreqCon[, 2]!="All")>0){
                    dpos <- x@intervals[[i]]@posFreqCon[x@intervals[[i]]@posFreqCon[, 2]!="All",]
                    for (unique in unique(as.numeric(as.character(dpos$Position)))) {
                        lines <- which(dpos$Position == unique)
                        start <- startlevel
                        for (k in lines) {
                            rect(splits[i] + unique-1/2, start,
                                 splits[i] + unique + 1/2, start +
                                 2*dpos$Lower[k], density=40,
                                 angle=-45, col="green")
                            c <- nchar(as.character(x@intervals[[i]]@posFreqCon$Nucleotide[k]))
                            l <- sum(int)
                            legend(splits[i] + unique-0.042-0.05760*l
                                   + 0.01996*c-0.008*l*c, start +
                                   dpos$Lower[k] + 0.06,
                                legend=dpos$Nucleotide[k], bty="n")
                            start <- start + 2*dpos$Lower[k]
                        }
                    }
                }
            }

        }

    }
})



## summary methods
setMethod("summary", signature(object="align"), function(object,...){
  df <- data.frame(seq=object@seq, pos=object@pos, orient=object@orient, motif=object@motif, prob=object@prob)
  cat("Motif sites:\n\n")
  cat("E-value:",object@eval,"\n\n")
  print(df)
})


setMethod("summary", signature(object="cosmo"), function(object,...){

  seqs <- object@seqs
  seqLengths <- NULL
  seqNames <- NULL
  for (i in 1:length(seqs)){
    seqLengths[i] <- nchar(seqs[[i]]$seq)
    seqNames[i] <- seqs[[i]]$desc
  }
  
  input <- data.frame(Sequence=seqNames,
                      Length=seqLengths) 
  cat("Input dataset:\n")
  print(input)
  if (!all(is.na(object@back$klDiv))){
    cat("\n\nCandidate orders for background Markov model:\n")
    print(object@back)
  }
  if (nrow(object@cand)>1){
    cat("\n\nCandidate models considered:\n")
    print(object@cand)
  }
  cat("\n\nSelected model:\n")
  print(object@sel)
  cat("\n\nSelected constraint set:\n")
  print(object@cons)
  cat("\n\nEstimated position weight matrix:\n")
  print(object@pwm)
  cat("\n")
  cat("Motif occurrences:\n\n")
  cat("E-value:",object@motifs@eval,"\n\n")
  print(object@motifs)

})

 

## print methods
 setMethod("print", signature(x="align"), function(x,...){
  df <- data.frame(seq=x@seq, pos=x@pos, orient=x@orient, motif=x@motif, prob=x@prob)
  print(df)
})

setMethod("print", signature(x="cosmo"), function(x,...){
  print(x@pwm)
})

setMethod("print", signature(x="constraintSet"), function(x,...){
  str <- writeConFile(x,stringOutput=TRUE)
  cat(str)
})




## show methods
setMethod("show", "align", function(object){
  df <- data.frame(seq=object@seq, pos=object@pos, orient=object@orient, motif=object@motif, prob=object@prob)
  show(df)
})

setMethod("show", "cosmo", function(object){
  show(object@pwm)
})

setMethod("show", "constraintSet", function(object){
  str <- writeConFile(object,stringOutput=TRUE)
  cat(str)
})



