makeConSet <- function(numInt, type, length, descrip="Constraint Set") {
    if (length(type)!=numInt | length(length)!=numInt ) {
      warning("arguments length and type must have same length equal to numInt")}
    
    intervals=lapply(1:numInt, function(x){
        if (type[x] == "B") {
            len <- as.numeric(length[x])
            prop <- as.numeric(NA)
        }
        if (type[x] == "V") {
            len <- as.numeric(NA)
            prop <- as.numeric(NA)
        }
        if (type[x] == "P") {
            len <- as.numeric(NA)
            prop <- as.numeric(length[x])
        }
        new("intInfo", constraintID=1, intervalID=x, type=type[x],
        length=len, prop=prop, boundedCon=data.frame(), posFreqCon=data.frame(),
        shapeCon=data.frame(), objectCall=match.call())
    })
    set=new("constraintSet", description=descrip, numIntervals=numInt,
      intervals=intervals, shapeDiffCon=data.frame(), subMotifCon=data.frame(),
      palCon=data.frame(), objectCall=match.call())
    set
}


makeBoundCon <- function(lower, upper) {
    new("boundCon", lower=lower, upper=upper)    
}

makePosFreqCon <- function(pos, nuc, lower) {
    new("posFreqCon", pos=pos, nuc=nuc, lower=lower)    
}

makeShapeCon <- function(shape, sLower, sUpper, eLower, eUpper, error) {
    new("shapeCon", shape=shape, sLower=sLower, sUpper=sUpper,
        eLower=eLower, eUpper=eUpper, error=error)    
}

makeShapeDiffCon <- function(int1, int2, lower, upper) {
    if (nchar(int1) == 2 & nchar(int2) == 2){
        if (is.na(as.numeric(substring(int1, 1, 1))) |
            is.na(as.numeric(substring(int2, 1, 1))) |
            !is.element(substring(int1, 2, 2), c("a", "b")) |
            !is.element(substring(int2, 2, 2), c("a", "b"))) {
            warning("Interval inputs for shape difference constraint must be a concatenation of the interval number and the extremity of that interval ('a' for beginning and 'b' for end)")            
        }
    }
    else {
        warning("Interval inputs for shape difference constraint must be a concatenation of the interval number and the extremity of that interval ('a' for beginning and 'b' for end)")
    }
    new("shapeDiffCon", int1=int1, int2=int2, lower=lower, upper=upper)    
}

makeSubMotifCon <- function(submotif, minfreq) {
    new("subMotifCon", submotif=submotif, minfreq=minfreq)    
}

makePalCon <- function(int1, int2, errBnd) {
    new("palCon", int1=int1, int2=int2, errBnd=errBnd)    
}

addCon <- function(conSet, constraint, int=1 ) {

    if (class(int) != "list") {int=as.list(int)}
    if (class(constraint) != "list")
      constraint <- list(constraint)

    if (length(int) != length(constraint)) {      
      warning("arguments int and constraint must have same length")}
    
        for (i in 1:length(int)) {
            if (class(constraint[[i]]) == "boundCon") {
                if (dim(conSet@intervals[[int[[i]]]]@boundedCon)[1] == 0) {
                    conSet@intervals[[int[[i]]]]@boundedCon <-
                      data.frame("Interval"=int[[i]],
                                 "Lower"=constraint[[i]]@lower,
                                 "Upper"=constraint[[i]]@upper)
                }
                else {
                    conSet@intervals[[int[[i]]]]@boundedCon <-
                      as.data.frame(t(cbind(t(conSet@intervals[[int[[i]]]]@boundedCon),
                                            c(int[[i]], constraint[[i]]@lower, constraint[[i]]@upper))))
                    rownames(conSet@intervals[[int[[i]]]]@boundedCon)<-
                      1:dim(conSet@intervals[[int[[i]]]]@boundedCon)[1]
                }
            }else if (class(constraint[[i]]) == "intFreqCon") {
                if (dim(conSet@intervals[[int[[i]]]]@intFreqCon)[1] == 0) {
                    conSet@intervals[[int[[i]]]]@intFreqCon <-
                      data.frame("Interval"=int[[i]],
                                 "Nucleotide"=constraint[[i]]@nuc,
                                 "Lower"=constraint[[i]]@lower)
                }
                else {
                    conSet@intervals[[int[[i]]]]@intFreqCon <-
                      as.data.frame(t(cbind(t(conSet@intervals[[int[[i]]]]@intFreqCon),
                                            c(int[[i]], constraint[[i]]@nuc, constraint[[i]]@lower))))
                    rownames(conSet@intervals[[int[[i]]]]@intFreqCon)<-
                      1:dim(conSet@intervals[[int[[i]]]]@intFreqCon)[1]
                    conSet@intervals[[int[[i]]]]@intFreqCon[, 1]<-
                      as.numeric(conSet@intervals[[int[[i]]]]@intFreqCon[, 1])
                    conSet@intervals[[int[[i]]]]@intFreqCon[, 2]<-
                      as.factor(conSet@intervals[[int[[i]]]]@intFreqCon[, 2])
                    conSet@intervals[[int[[i]]]]@intFreqCon[, 3]<-
                      as.numeric(conSet@intervals[[int[[i]]]]@intFreqCon[, 3])
                }
            }else if (class(constraint[[i]]) == "posFreqCon") {
                if (dim(conSet@intervals[[int[[i]]]]@posFreqCon)[1] == 0) {
                    conSet@intervals[[int[[i]]]]@posFreqCon <-
                      data.frame("Interval"=int[[i]],
                                 "Position"=constraint[[i]]@pos,
                                 "Nucleotide"=constraint[[i]]@nuc,
                                 "Lower"=constraint[[i]]@lower)
                }
                else {
                    conSet@intervals[[int[[i]]]]@posFreqCon <-
                      as.data.frame(t(cbind(t(conSet@intervals[[int[[i]]]]@posFreqCon),
                                            c(int[[i]], constraint[[i]]@pos,
                                              constraint[[i]]@nuc, constraint[[i]]@lower))))
                    rownames(conSet@intervals[[int[[i]]]]@posFreqCon)<-
                      1:dim(conSet@intervals[[int[[i]]]]@posFreqCon)[1]
                    conSet@intervals[[int[[i]]]]@posFreqCon[, 1]<-
                      as.numeric(conSet@intervals[[int[[i]]]]@posFreqCon[, 1])
                    conSet@intervals[[int[[i]]]]@posFreqCon[, 2]<-
                      as.factor(conSet@intervals[[int[[i]]]]@posFreqCon[, 2])
                    conSet@intervals[[int[[i]]]]@posFreqCon[, 3]<-
                      as.factor(conSet@intervals[[int[[i]]]]@posFreqCon[, 3])
                    conSet@intervals[[int[[i]]]]@posFreqCon[, 4]<-
                      as.numeric(as.character(conSet@intervals[[int[[i]]]]@posFreqCon[, 4]))
                }
            }else if (class(constraint[[i]]) == "shapeCon") {
                if (dim(conSet@intervals[[int[[i]]]]@shapeCon)[1] == 0) {
                    conSet@intervals[[int[[i]]]]@shapeCon <-
                      data.frame("Interval"=int[[i]],
                                 "Shape"=constraint[[i]]@shape,
                                 "StartLower"=constraint[[i]]@sLower,
                                 "StartUpper"=constraint[[i]]@sUpper,
                                 "EndLower"=constraint[[i]]@eLower,
                                 "EndUpper"=constraint[[i]]@eUpper,
                                 "Error"=constraint[[i]]@error)
                }
                else {
                    conSet@intervals[[int[[i]]]]@shapeCon <-
                      as.data.frame(t(cbind(t(conSet@intervals[[int[[i]]]]@shapeCon),
                                            c(int[[i]], constraint[[i]]@shape,
                                              constraint[[i]]@sLower,
                                              constraint[[i]]@sUpper,
                                              constraint[[i]]@eLower,
                                              constraint[[i]]@eUpper,
                                              constraint[[i]]@error))))
                    rownames(conSet@intervals[[int[[i]]]]@posFreqCon)<-
                      1:dim(conSet@intervals[[int[[i]]]]@shapeCon)[1]
                    conSet@intervals[[int[[i]]]]@shapeCon[, 1]<-
                      as.numeric(conSet@intervals[[int[[i]]]]@shapeCon[, 1])
                    conSet@intervals[[int[[i]]]]@shapeCon[, 2]<-
                      as.numeric(conSet@intervals[[int[[i]]]]@shapeCon[, 2])
                    conSet@intervals[[int[[i]]]]@shapeCon[, 3]<-
                      as.factor(conSet@intervals[[int[[i]]]]@shapeCon[, 3])
                    conSet@intervals[[int[[i]]]]@shapeCon[, 4]<-
                      as.numeric(conSet@intervals[[int[[i]]]]@shapeCon[, 4])
                }
            }else if (class(constraint[[i]]) == "subMotifCon") {
                if (dim(conSet@subMotifCon)[1] == 0) {
                    conSet@subMotifCon <- data.frame("Constraint"=1,
                                                     "Submotif"=constraint[[i]]@submotif,
                                                     "MinFreq"=constraint[[i]]@minfreq)
                }
                else {
                    conSet@subMotifCon <- rbind(conSet@subMotifCon,
                                                c(1, constraint[[i]]@submotif, constraint[[i]]@minfreq))
                    rownames(conSet@subMotifCon)<- 1:dim(conSet@subMotifCon)[1]
                }
                conSet@subMotifCon[, 1]<- as.numeric(conSet@subMotifCon[, 1])
                conSet@subMotifCon[, 2]<- as.character(conSet@subMotifCon[, 2])
                conSet@subMotifCon[, 3]<- as.numeric(conSet@subMotifCon[, 3])

            }else if (class(constraint[[i]]) == "shapeDiffCon") {
                if (dim(conSet@shapeDiffCon)[1] == 0) {
                  conSet@shapeDiffCon <- data.frame("Constraint"=1,
                                                    "Interval1"=constraint[[i]]@int1,
                                                    "Interval2"=constraint[[i]]@int2,
                                                    "Lower"=constraint[[i]]@lower,
                                                    "Upper"=constraint[[i]]@upper)
                }
                else {
                    conSet@shapeDiffCon <- rbind(conSet@shapeDiffCon,
                                                c(1, constraint[[i]]@int1, constraint[[i]]@int2,
                                                  constraint[[i]]@lower, constraint[[i]]@upper))
                    rownames(conSet@shapeDiffCon)<- 1:dim(conSet@shapeDiffCon)[1]
                }
                conSet@shapeDiffCon[, 1]<- as.numeric(conSet@shapeDiffCon[, 1])
                conSet@shapeDiffCon[, 2]<- as.character(conSet@shapeDiffCon[, 2])
                conSet@shapeDiffCon[, 3]<- as.character(conSet@shapeDiffCon[, 3])
                conSet@shapeDiffCon[, 4]<- as.numeric(conSet@shapeDiffCon[, 4])
                conSet@shapeDiffCon[, 5]<- as.numeric(conSet@shapeDiffCon[, 5])

            }else if (class(constraint[[i]]) == "palCon") {
                if (dim(conSet@palCon)[1] == 0) {
                    conSet@palCon <- data.frame("Constraint"=1,
                                                "Interval1"=constraint[[i]]@int1,
                                                "Interval2"=constraint[[i]]@int2,
                                                "ErrorBound"=constraint[[i]]@errBnd)
                }
                else {
                    conSet@palCon <- rbind(conSet@palCon,
                                           c(1, constraint[[i]]@submotif,
                                             constraint[[i]]@int2, constraint[[i]]@errBnd))
                    rownames(conSet@palCon)<- 1:dim(conSet@palCon)[1]
                }
                conSet@palCon[, 1]<- as.numeric(conSet@palCon[, 1])
                conSet@palCon[, 2]<- as.numeric(conSet@palCon[, 2])
                conSet@palCon[, 3]<- as.numeric(conSet@palCon[, 3])
                conSet@palCon[, 4]<- as.numeric(conSet@palCon[, 4])
            }else{
              stop(paste("Class", class(constraint[[i]]),"of constraint", i, "not recognized."))
            }

        }

    conSet
    
}


readConFile <- function(conFile=NULL,
                        description="Constraints"){ 
    if (is.null(conFile)) {
      conFile <-
        fileBrowser(textToShow="Path to constraint set(s)")
    } 
    
    if (class(conFile) != "character")
      warning("conFile argument must be of class character (path to constraint set(s))")
    
    if (!file.exists(conFile))
      stop(paste("Constraint file", conFile, "not found.\n"))

    rl <- readLines(conFile)
    l <- length(rl)
    collapse <- function(x){gsub(" ", "", x)}
    catch <- function(string, data, element){
             bound <- unlist(lapply(strsplit(data[grep(paste(string,
                            ":", sep=""), data)], split=" "), 
                            function(x){collapse(x[element])}) ) 
           bound    
    }
    spacecatch <- function(string, element){
             strsplit(string, " ")[[1]][element]    
    }
    anycatch <- function(string, afterwhat){
        strsplit(string, afterwhat)[[1]][2]
    }
    between <- function(number, sorted){
        sum(sorted<=number)
    }
    conStart <- grep("@ ConstraintSet", rl)
    intsetup <- grep(">IntervalSetup", rl)
        conBounds <- c(conStart, l + 1)
        
    constraintSets <- list()

    for (i in 1:length(conStart)){

        ##INTSETUP
 
        first <- intsetup[i] + 1
        type <- c()
        length <- c()
        prop <- c()
        while(substring(rl[first], 1, 7) == "Length:"){
            if (anycatch(rl[first],  "Length: ") == "variable"){
                type <- c(type, "V")
                length <- c(length, NA)
                prop <- c(prop, NA)
            }
            else if (length(grep("%", anycatch(rl[first], "Length: ")))!=0){
                type <- c(type, "P")
                length <- c(length,
                            as.numeric(substring(anycatch(rl[first], "Length: "), 1,
                                                 nchar(anycatch(rl[first], "Length: "))-1))) 
            }
            else {
                type <- c(type, "B")
                length <- c(length, as.numeric(spacecatch(rl[first], 2)))
            }
            first <- first + 1
        }

        constraintSet <- makeConSet(length(type), type, length,
                                    descrip="Constraint Set") 

        ## Submotif

        lines <- grep(">SubMotif", rl)
        lines <- lines[lines>=conBounds[i] & lines < conBounds[i + 1]]
        if (length(lines)>0){
            for (k in 1:length(lines)){
              SMC <- makeSubMotifCon(anycatch(rl[lines[k] + 1],
                                              "Motif: "), as.numeric(anycatch(rl[lines[k] + 2],
                                                                              "MinFreq: "))) 
                constraintSet <- addCon(constraintSet, list(SMC), 1) 
            }
        }

        ## Palindrome

        lines <- grep(">Pal", rl)
        lines <- lines[lines>=conBounds[i] & lines < conBounds[i + 1]]
        if (length(lines)>0){
            for (k in 1:length(lines)){
                PC <- makePalCon(as.numeric(spacecatch(rl[lines[k] + 1], 2)),
                                 as.numeric(spacecatch(rl[lines[k] + 1], 4)),
                                 as.numeric(spacecatch(rl[lines[k] + 2], 2))) 
                constraintSet <- addCon(constraintSet, list(PC), 1)
            }
        }

        ## Shape Diff

        lines <- grep(">ParmDiff", rl)
        lines <- lines[lines>=conBounds[i] & lines < conBounds[i + 1]]
        if (length(lines)>0){
            for (k in 1:length(lines)){
                SDC <- makeShapeDiffCon(as.numeric(substring(1, 1,spacecatch(rl[lines[k] + 1], 2))),
                                        as.numeric(substring(1, 1, spacecatch(rl[lines[k] + 1], 4))),
                    as.numeric(spacecatch(rl[lines[k] + 2], 2)), as.numeric(spacecatch(rl[lines[k] + 2], 2)))
                constraintSet <- addCon(constraintSet, list(SDC), 1)
            }
        }

        ## IcBounds

        lines <- grep(">IcBounds", rl)
        lines <- lines[lines>=conBounds[i] & lines < conBounds[i + 1]]
        if (length(lines)>0){
            for (k in 1:length(lines)){
                ICBC <- makeBoundCon(as.numeric(spacecatch(rl[lines[k] + 2],2)),
                                     as.numeric(spacecatch(rl[lines[k] + 2], 4)))
                constraintSet <- addCon(constraintSet, list(ICBC), 
                                        as.numeric(spacecatch(rl[lines[k] + 1], 2)))
            }
        }

        ## IcShape

        lines <- grep(">IcShape", rl)
        lines <- lines[lines>=conBounds[i] & lines < conBounds[i + 1]]
        if (length(lines)>0){
            for (k in 1:length(lines)){
                ICSC <- makeShapeCon(anycatch(rl[lines[k] + 2], "Shape: "),
                                     as.numeric(spacecatch(rl[lines[k] + 3], 2)),
                                     as.numeric(spacecatch(rl[lines[k] + 3], 4)),
                                     as.numeric(spacecatch(rl[lines[k] + 4], 2)),
                                     as.numeric(spacecatch(rl[lines[k] + 4], 4)),
                                     as.numeric(spacecatch(rl[lines[k] + 5], 2)))
                constraintSet <- addCon(constraintSet, list(ICSC),
                                        as.numeric(spacecatch(rl[lines[k] + 1], 2)))
            }
        }

        ## NucFreq

        lines <- grep(">NucFreq", rl)
        lines <- lines[lines>=conBounds[i] & lines < conBounds[i + 1]]
        if (length(lines)>0){
            for (k in 1:length(lines)){
                NFC <- makePosFreqCon(spacecatch(rl[lines[k] + 2], 2),
                                      spacecatch(rl[lines[k] + 3], 2),
                                      as.numeric(spacecatch(rl[lines[k] + 4], 2)))
                constraintSet <- addCon(constraintSet, list(NFC),
                                        as.numeric(spacecatch(rl[lines[k] + 1], 2)))
            }
        }

        constraintSets[[i]]<- constraintSet

    }

    if (length(constraintSets) == 1){
      constraintSets[[1]]
    }else{
      constraintSets
    }

}

writeConFile <-
  function(constraints, stringOutput=TRUE, outputFile="constraints.txt"){

    
    if (class(constraints) == "list"){
      if (!all(unlist(lapply(constraints, function(x) class(x)[1])) ==
               "constraintSet")){
        stop("If constraints is a list, all elements must be of class constraintSet.")
      }
    }else if (class(constraints) == "constraintSet") {
      constraints <- list(constraints)
    }else{
      stop("constraints must be of class constraintSet or a list whose elements are of class constraintSet")
    }

    numSets <- length(constraints)
    
    text <- NULL
    for (i in 1:numSets) {

        text<-rbind(text, paste("@ ConstraintSet:", i), "", ">IntervalSetup")


        ## Interval-specific constraints

        for (j in 1:constraints[[i]]@numIntervals) {
            if (constraints[[i]]@intervals[[j]]@type == "B"){
                text<-rbind(text,
                            paste("Length: ",
                                  constraints[[i]]@intervals[[j]]@length,
                                  " bp", sep=""))
            }
            else if (constraints[[i]]@intervals[[j]]@type == "P"){
                text<-rbind(text, paste("Length: ",
                                        constraints[[i]]@intervals[[j]]@prop,
                                        "%", sep=""))
            }
            else if (constraints[[i]]@intervals[[j]]@type == "V"){
                text<-rbind(text, "Length: variable")
            }
        }

        text<-rbind(text, "")

        for (j in 1:constraints[[i]]@numIntervals) {

            if (dim(constraints[[i]]@intervals[[j]]@boundedCon)[1]!=0) {
                for (k in 1:nrow(constraints[[i]]@intervals[[j]]@boundedCon)) {
                    text<-rbind(text, ">IcBounds",
                    paste("Interval: ", j, sep=""),
                    paste("Bounds: ",
                        constraints[[i]]@intervals[[j]]@boundedCon$Lower[k], " to ",
                        constraints[[i]]@intervals[[j]]@boundedCon$Upper[k], "", sep=""), "")
                }
            }

            if (dim(constraints[[i]]@intervals[[j]]@posFreqCon)[1]!=0) {
                for (k in 1:nrow(constraints[[i]]@intervals[[j]]@posFreqCon)) {
                    text<-rbind(text, ">NucFreq",
                    paste("Interval: ", j, sep=""),
                    paste("Pos: ",
                          constraints[[i]]@intervals[[j]]@posFreqCon$Position[k], sep=""),
                    paste("Nuc: ",
                          constraints[[i]]@intervals[[j]]@posFreqCon$Nucleotide[k], sep=""),
                    paste("LowerBound: ",
                          constraints[[i]]@intervals[[j]]@posFreqCon$Lower[k], sep=""),
                                "")
                }
            }    

            if (dim(constraints[[i]]@intervals[[j]]@shapeCon)[1]!=0) {
                for (k in 1:dim(constraints[[i]]@intervals[[j]]@shapeCon)[1]) {
                    text<-rbind(text, ">IcShape",
                    paste("Interval: ", j, sep=""),
                    paste("Shape:",
                          constraints[[i]]@intervals[[j]]@shapeCon$Shape[k]),
                    paste("LeftBounds: ",
                          constraints[[i]]@intervals[[j]]@shapeCon$StartLower[k],
                          " to ",
                          constraints[[i]]@intervals[[j]]@shapeCon$StartUpper[k],
                          sep=""),
                    paste("RightBounds: ",
                          constraints[[i]]@intervals[[j]]@shapeCon$EndLower[k],
                          " to ",
                          constraints[[i]]@intervals[[j]]@shapeCon$EndUpper[k], sep=""),
                    paste("ErrorTol:", constraints[[i]]@intervals[[j]]@shapeCon$Error[k]), "")
                }
            }

        }

        ## Global constraints        

        if (dim(constraints[[i]]@shapeDiffCon)[1]!=0) {
            for (k in 1:dim(constraints[[i]]@shapeDiffCon)[1]) {
                int1 <- constraints[[i]]@shapeDiffCon$Interval1[k]
                int2 <- constraints[[i]]@shapeDiffCon$Interval2[k]                
                lower <- constraints[[i]]@shapeDiffCon$Lower[k]
                upper <- constraints[[i]]@shapeDiffCon$Upper[k]                
                text <- rbind(text, ">ParmDiff",
                              paste("Parameters: ", int1, " - ", int2, sep=""),
                              paste("Bounds: ", lower, " to ", upper, sep=""), "") 
            }
        }

        if (dim(constraints[[i]]@palCon)[1]!=0) {
            for (k in 1:dim(constraints[[i]]@palCon)[1]) {
                text<-rbind(text, ">Pal",
                paste("Intervals: ",
                      constraints[[i]]@palCon$Interval1[k],
                      " and ", constraints[[i]]@palCon$Interval2[k], sep=""),
                paste("ErrorTol: ", constraints[[i]]@palCon$ErrorBound[k], sep=""), "")
            
            }
        }

        if (dim(constraints[[i]]@subMotifCon)[1]!=0) {
            for (k in 1:dim(constraints[[i]]@subMotifCon)[1]) {
                text<-rbind(text, ">SubMotif",
                paste("Motif: ", constraints[[i]]@subMotifCon$Submotif[k], sep=""),
                paste("MinFreq: ", constraints[[i]]@subMotifCon$MinFreq[k], sep=""), "")
            }
        }    

    }

    if (stringOutput){
        text<-rbind(text, "")
        text <- paste(text[text!=""], collapse="\n")
                paste(text, "\n", sep="")
    }
    else{
        write.table(text, file=outputFile, quote=FALSE, col.names=FALSE, row.names=FALSE, append=TRUE)
        outputFile
    }

}
