\name{constraintSet-class}
\docType{class}
\alias{constraintSet-class}
\alias{plot,constraintSet-method}
\alias{print,constraintSet-method}
\alias{show,constraintSet-method}

\title{Class "constraintSet" ~~~ }
\description{ An object of class \code{"constraintSet"} regroups all the
  constraints one wishes to impose when performing detection. The object
  consits essentially of constraints on the motif broken down into
  seperate intervals, objects of class \code{"intInfo"}}

\section{Objects from the Class}{
Objects can be created by calls of the form 
\code{  new('constraintSet',}\cr
\code{    description      = ...., # Object of class \code{"character"}}\cr
\code{    numIntervals     = ...., # Object of class \code{"numeric"}}\cr
\code{    intervals        = ...., # Object of class \code{"list"}}\cr
\code{    shapeDiffCon     = ...., # Object of class \code{"data.frame"}}\cr
\code{    subMotifCon      = ...., # Object of class \code{"data.frame"}}\cr
\code{    palCon           = ...., # Object of class \code{"data.frame"}}\cr
\code{    objectCall       = ...., # Object of class \code{"call"}}\cr
\code{  )}
}
\section{Slots}{
  \describe{
    \item{\code{description}:}{Object of class \code{"character"} a general description of the constraint set }
    \item{\code{numIntervals}:}{Object of class \code{"numeric"} number of intervals the motif was split into }
    \item{\code{intervals}:}{Object of class \code{"list"} a list of
      objects of class \code{"intInfo"} each describing the constraints
      imposed on the individual intervals}    
    \item{\code{shapeDiffCon}:}{Object of class \code{"data.frame"} a
      data frame summarizing the global constraints of the shape
      constraints. The data frame has four columns: the first two
      columns are the intervals and the extremity subject to the
      constraint, the extremity being supplied as 'a' for beginning, and 'b' for
      end. The third and fourth column are respectively the lower and
      upper bounds on the difference between the interval extremities
      considered. For example, if c("1b","2a",-0.2,0.5) were a row in
      the data frame, it would indicate that the difference in the information content
      between the end of the first interval and the beginning of the
      second should be between -0.2 and 0.5.}    
    \item{\code{subMotifCon}:}{Object of class \code{"data.frame"} a
      data frame specifying the submotif global constraints. The two
      columns respectiuvely indicate the submotif of interest and it's
      error tolerance. The error is computed as the difference in terms      
      of frequency between the submotif and the position weight matrix.}    
    \item{\code{palCon}:}{Object of class \code{"data.frame"} a data
      frame summarizing the palindromic constraints imposed on the
      motif: the first two columns indicate the two palindromic
      intervals and the third is the error tolerated between the
      nucleotide frequencies in both intervals. }    
    \item{\code{objectCall}:}{Object of class \code{"call"} stores how
      the object was created }    
  }
}
\section{Methods}{
\describe{
  \item{plot}{\code{signature(x = "constraintSet", varLen=4,
      propLen=4, plot.IC=TRUE, plot.nucFreq=TRUE)}: Plots the
    \code{"constraintSet"} object. varLen and propLen are integers
    indicating how wide to make the variable and proportion intervals (1
    unit is one base pair).    
    plot.IC and plot.nucFreq are logicals respectively indicating
    whether the IC content and the nucleotide frequencies are to be
    drawn on the plot.}

  \item{print}{\code{signature(x = "constraintSet",...)} Prints the
    constraint set in the standard cosmo format.}

  \item{show}{\code{signature(object = "constraintSet")} Prints the
    constraint set in the standard cosmo format.}

  
  }
}
\references{ }
\author{
Fabian Gallusser, \email{fgallusser@berkeley.edu}
}
\note{}
\seealso{
  \code{\link{writeConFile}}, 
  \code{\link{constraintSet-class}},
  \code{\link{intInfo-class}}
}
\examples{
}
\keyword{classes}
