\name{intInfo-class}
\docType{class}
\alias{intInfo-class}
\alias{intInfo}
\alias{plot,intInfo-method}

\title{Class "intInfo" ~~~ }
\description{ An object of class \code{"intInfo"} contains all the
  constraints one wishes to impose on an interval of the motif.  
  Combined \code{"intInfo"} objects will constitue an object of class
  \code{"constraintSet"}}

\section{Objects from the Class}{Objects can be created by calls of the form 
\code{  new('intInfo',}\cr
\code{    constraintID      = ...., # Object of class \code{"numeric"}}\cr
\code{    intervalID        = ...., # Object of class \code{"numeric"}}\cr
\code{    type              = ...., # Object of class \code{"character"}}\cr
\code{    length            = ...., # Object of class \code{"numeric"}}\cr
\code{    prop              = ...., # Object of class \code{"numeric"}}\cr
\code{    boundedCon        = ...., # Object of class \code{"data.frame"}}\cr
\code{    posFreqConCon     = ...., # Object of class \code{"data.frame"}}\cr
\code{    shapeCon          = ...., # Object of class \code{"data.frame"}}\cr
\code{    objectCall        = ...., # Object of class \code{"call"}}\cr
\code{  )}
}
\section{Slots}{
  \describe{
    \item{\code{constraintID}:}{Object of class \code{"numeric"},
      indicating which constraint set the interval is part of}    
    \item{\code{intervalID}:}{Object of class \code{"numeric"},
      indicating the order of the interval in the constraint set}    
    \item{\code{type}:}{Object of class \code{"character"}, the interval
      type: 'B' for base pairs, 'P' for proportion, or 'V' for variable}
    \item{\code{length}:}{Object of class \code{"numeric"}, the length
      of the interval: if the interval is of type 'B', the length is the
      number of base pairs, if the      
      interval is of type 'P' or 'V', this slot is NA}    
    \item{\code{prop}:}{Object of class \code{"numeric"}, the ratio of
      the interval lengtb to the total motif width. This slot is only
      defined for intervals of type 'P', in which case      
      the value is between 0 and 100\%. For intervals of type 'B' or
      'V', this slot is NA}    
    \item{\code{boundedCon}:}{Object of class \code{"data.frame"} a data
      frame summarizing the information content bounding constraints on
      the interval: the first column indicates the position of the
      interval among the intervals constituing the constraint set, the
      second and third column are respetively the lower and upper bound
      of the information content on the interval.}
    \item{\code{posFreqCon}:}{Object of class \code{"data.frame"} a data
      frame summarizing the position nucleotide frequency constraints
      imposed on the interval: the first column indicates the interval
      to which the constraint is applied, the second column the position
      concerned ('All' cab be specified if the constraint applies to the
      entire interval). It is to be noted that a specific position mat
      only be specified if the interval is of type 'B'. The third column
      is the nucleotide concerned, either A, C, G, T, AT, or GC are
      accepted at this time. Finally the fourth column indicates the
      lower bound for the given nucleotide at the given position. }    
    \item{\code{shapeCon}:}{Object of class \code{"data.frame"} a data
      frame summarizing the shape constraint on the interval. The first
      column indicates the interval to which the constraint is applied,
      the second the shape of the variation either 'linear', 'monotone
      decreasing', or 'monotone increasing'. The third and fourth column
      (respectively fifth and sixth) record the lower and upper bounds
      of the information content at the beginning (respectively end) of
      the interval. Finally, the seventh column records the error
      tolerated.}    
    \item{\code{objectCall}:}{Object of class \code{"call"} stores how
      the object was created }    
  }
}
\section{Methods}{
\describe{
  \item{plot}{\code{signature(x = "intInfo", varLen=4, propLen=4,
      plot.IC=TRUE, plot.nucFreq=TRUE)}: plots the \code{"intInfo"}
    object. varLen and propLen are integers indicating how wide to make
    the variable and proportion intervals (1 unit is one base pair).    
    plot.IC and plot.nucFreq are logicals respectively indicating
    whether the IC content and the nucleotide frequencies are to be
    drawn on the plot.}
  
  }
}
\references{ }
\author{
Fabian Gallusser, \email{fgallusser@berkeley.edu}
}
\note{}
\seealso{
  \code{\link{writeConFile}},
}
\examples{
}
\keyword{classes}


