\name{simScore}
\alias{simScore}
\title{Score motif detection simulation results}
\description{
  This function computes sensitivity and specifity for the results
  returned by cosmo.
}
\usage{
  simScore(truth, cosmoOut, minOverlap=0.25)
}
\arguments{
  \item{truth}{\code{align} Alignment describing the true motif occurrences.}
  \item{cosmoOut}{\code{cosmo} The results returned by \code{cosmo()}.}
  \item{minOverlap}{\code{numeric} A predicted motif must overlap at least
    this proportion of a known motif to be considered a hit.}
}
\details{
}
\value{
  \item{sens}{The proportion of true motif occurrences discovered
    (sensitivity).}
  \item{spec}{The proportion of true motif occurrences among the
    discovered sites (specificity).}
  \item{roc}{The area under the ROC curve.}
}
\references{}
\author{Oliver Bembom, \email{bembom@berkeley.edu}}
\note{}

\seealso{ \code{\link{cosmo}} }
\examples{
## generate 20 sequences according to OOPS model
## with an expected 50% of sequences containing a
## motif
data(motifPWM)
data(transMats)
res <- rseq(20, 100, 1.0, motifPWM, transMats,"ZOOPS")
truth <- res$motifs
seqs <- res$seqs

res <- cosmo(seqs, constraints="None", minW=8, maxW=8)
simScore(truth, res)

}
\keyword{misc}
