/* various macros for indexing into arrays */
#define LR(w,n,l,p)   (((w)-dataSet->minWidth)*dataSet->numSeqs*dataSet->maxSeqLength*nnSites + (n)*dataSet->maxSeqLength*nnSites + \
                        (l)*nnSites + (p))
#define PWM(p,n)      ((p)*4 + (n))
#define CPB(w,s)      (((w) - minWidth)*seqLength + (s))
#define LCM(i,j)      ((i)*(mod->n) + (j))
#define RS(s)         (seqLength - width - (s))
#define PP(n,l)       ((n)*maxSeqLength + (l))
#define SV(w,c,i,s)   (((w) - dataSet->minWidth)*dataSet->numConSets*nnSites*numSv + (c)*nnSites*numSv + (i)*numSv + (s))
#define MOD(w,m,c)    ((w) - dataSet->minWidth)*dataSet->numModTypes*dataSet->numConSets + (m)*dataSet->numConSets + (c)
#define II(i,j)       ((i)*(N+1) + (j))
#define LL(i,j)       ((i)*(N+1) + (j))
#define SS(i,j)       ((i)*(Irange+1) + (j))

/* Useful data types and constants */
typedef int BOOLEAN;
#define FALSE 0
#define TRUE 1


/* 
   debugging level
   for checkcon, this needs to be set to 0 
   by using -DDEBUG=0 in the compilation stage

#ifdef DEBUG
#else
#define DEBUG 2
#endif
*/
    
/* maximum number of characters to read in at once from constraint file */
#define MAXNUMCHARS 100

extern int COSMO_MSG_LEVEL;  


/* model types */
typedef enum {OOPS, ZOOPS, TCM, OVER, CUT} MTYPE;

/* nucleotides */
typedef enum {A,C,G,T,AT,GC} NUC; 

/* types of interval lengths */
typedef enum {bp,prop,var} LENTYPE;

/* start or end of interval */
typedef enum {start, end} POS;

/* type of nonlinear constraints */
typedef enum {lin,monInc,monDec,bdd,sub} CONTYPE;

/* reference point for monotone constraint */
typedef enum {prevPos, prevBrk, nextBrk} REF;

/* names of model selection criteria */
typedef enum {lik, aic, bic, eval, likCV, pwmCV} CTYPE;


/* probability distribution */ 
typedef struct distr {
  int w;			/* maximum width of alignment */
  double alpha;			/* scale factor for distribution */
  int *range;			/* range of distributions w = [1..w] */
  int *offset;			/* offset of distributions w = [1..w] */
  double *d;			/* array of (log) distributions w = 1 */
  double *cdf;			/* array of llr (log) 1 - CDFs w = 1 */
  double mean;			/* mean of llr for w = 1 */
} DISTR;

/* constraint file in strings*/
typedef struct conString{
  char *con;                      /* array of strings that make up lines of constraint file */
  int nlines;                      /* number of lines in constraint file = number of strings in array */
  int index;                       /* current index into array */
} CONSTRING;


/* a model selection criterion definition*/
typedef struct critDef{
  char *name;                      /* the name of the criterion in a string*/
  CTYPE crit;                      /* the name of the criterion as a CRIT */
  int fold;                        /* the fold to use for CV, if applicable */
  BOOLEAN trunc;                   /* truncate loss function in likelihood-based CV? */
  double quant;                    /* the truncation quantile q \in [0,1] */
} CRITDEF;


/* to keep track of best model so far */
typedef struct criterion{
  BOOLEAN intInit;                 /* has the intensity criterion been initialized? */
  int     startNum;                /* the best starting value so far */
  int     nSitesNum;               /* index of best number of sites so far */
  double  intCrit;                 /* best criterion for choosing intensity parameter so far */

  BOOLEAN wInit;                   /* has the width criterion been initialized? */
  int     width;                   /* best width so far */
  double  wCrit;                   /* best width criterion so far */
  
  BOOLEAN modInit;                 /* has the model criterion been initialized? */
  int     modTypeNum;              /* index of best model type so far */
  MTYPE   mType;                   /* model type */
  double  modCrit;                 /* best model criterion so far */

  BOOLEAN conInit;                 /* has the constraint set criterion been initialized? */
  int     conSetNum;               /* best constraint set so far */
  double  conCrit;                 /* best constraint set criterion so far */
} CRIT;


/* an interval frequency constraint */
typedef struct IntFreqCon{          
  double low;                      /* the lower bound on a nucleotide frequency */
  NUC nuc;                         /* the nucleotide whose frequency is to be bounded */
  struct IntFreqCon *next;         /* pointer to the next interval frequency constraint */
} INTFREQCON;


/* a position frequency constraint */
typedef struct PosFreqCon{
  double low;                      /* the lower bound on a nucleotide frequency */
  int pos;                         /* the position in the interval to which this bound applies to */
  NUC nuc;                         /* the nucleotide whose frequency is to be bounded */
  struct PosFreqCon *next;         /* pointer to the next position frequency constraint */
} POSFREQCON;

/* a palindrome constraint */
typedef struct PalCon{
  int int1, int2;                  /* the intervals that are assumed to be palindromes of each other */
  double error;                    /* an error tolerance for how different nucleotide frequencies are allowed to be */
} PALCON;


/* a constraint on the shape of the information content profile across an interval */
typedef struct ShapeCon{
  CONTYPE shape;                   /* the shape: linear, monotone increasing, monotone decreasing */
  double startLow;                 /* lower bound on IC at beginning of interval */
  double startUp;                  /* upper bound on IC at beginning of interval */
  double endLow;                   /* lower bound on IC at end of interval */
  double endUp;                    /* upper bound on IC at end of interval */
  double error;                    /* error tolerance for how much the specified shape may be deviated from */
  int number;                      /* counter for shape constraints */
} SHAPECON;

/* a bound constraint on the information content profile across an interval */
typedef struct BoundCon{
  double low;                      /* lower bound on the IC across the interval */
  double up;                       /* upper bound on the IC across the interval */
} BOUNDCON;

/* a submotif constraint */
typedef struct SubCon{
  CONTYPE conType;                 /* type of constraint: here submotif constraint */
  int length;                      /* the length of the submotif */
  NUC *motif;                      /* the sequence of nucleotides making up the motif */
  double cutOff;                   /* roughly, the minimum proportion with which these nucleotides must occur */
} SUBCON;

/* collection of constraints on a given interval */
typedef struct IntCon{

  LENTYPE lenType;                 /* how the length of the interval is defined: fixed number of bp, proportional, etc. */
  double prop;                     /* the proportion of the whole motif comprised by this interval (if applicable) */
  int numBasePairs;                /* the length of the interval in base pairs (if applicable) */

  int hasBoundCon;                 /* does the interval have a bound constraint? */
  BOUNDCON boundCon;               /* a bound constraint */

  int hasShapeCon;                 /* does the interval have a shape constraint? */
  SHAPECON shapeCon;               /* a shape constraint */

  int numIntFreqCons;              /* number of interval frequency constraints this interval has */
  INTFREQCON *intFreqCons;         /* pointer to the first interval frequency constraint */
    
  int numPosFreqCons;              /* number of position frequency constraints this interval has */
  POSFREQCON *posFreqCons;         /* pointer to the first position frequency constraint */
} INTCON;


/* Linear constraint on infomation content parameters */
typedef struct LinCon{
  POS pos1;                        /* is parameter 1 at the start or end of its interval? */
  POS pos2;                        /* is parameter 2 at the start or end of its interval? */
  int int1;                        /* the interval to which parameter 1 belongs */
  int int2;                        /* the interval to which parameter 2 belongs */
  int index1;                      /* the index of parameter 1 */ 
  int index2;                      /* the index of parameter 2 */
  double low;                      /* lower bound for difference parm1 - parm2 */
  double up;                       /* upper bound for difference parm1 - parm2 */
  struct LinCon *next;             /* pointer to next linear constraint on IC parameters */
} LINCON;

/* the collection of constraints on the PWM */
typedef struct ConSet{
    
  int numInts;                     /* the number of intervals */
  int numShapeCons;                /* the number of shape constraints */ 
  int numBoundCons;                /* the number of bound constraints */
  int numIntFreqCons;              /* the number of interval frequency constraints */
  INTCON *intCons;                 /* pointer to the first interval frequency constraint */

  int hasPalCon;                   /* is there a palindrome constraint? */
  PALCON palCon;                   /* a palindrome constraint */

  int hasSubCon;                   /* is there a submotif constraint? */
  SUBCON subCon;                   /* a submotif constraint */
 
  int numLinCons;                  /* number of linear constraints on IC parameters */
  LINCON *linCons;                 /* pointer to first linear constraint on IC parameters */

  int widthLow;                    /* the minimum width of the motif to look for */
  double sharpestBound;            /* the max of any lower bounds that are given for nucleotide frequencies */
 
  BOOLEAN isEmpty;                 /* does the constraint set not contain any constraints? */
} CONSET;

 
/* a constraint on the information content in a particular position of an interval*/ 
typedef struct ProfCon{
  CONTYPE conType;                  /* type of constraint: lin, monInc, monDec, bdd */ 
  REF ref;                          /* reference position that IC at this position is to be compared to */
  int pos;                          /* position of interest */
  int interval;                     /* interval that this position is in */
  double xPos;                      /* position in interval measured on scale from 0.0 to 1.0 */
  int parm1;                        /* parameter for IC at the beginning of this interval */
  int parm2;                        /* parameter for IC at the end of this interval  */
} PCON;
 

/* a nonlinear constraint on the set of parameters to estimate */
typedef union NlCon{
  PCON pCon;                        /* a constraint on the IC in a particular position of an interval */ 
  SUBCON subCon;                    /* a submotif constraint */
}NLCON;

/* a constraint set that has been interpreted for a given width */
typedef struct Model{
  MTYPE mType;                      /* the model type: OOPS, ZOOPS, etc. */
    
  int width;                        /* the current width of the motif */
  int n;                            /* number of parameters to estimate */
  int nlin;                         /* number of linear constraints */
  int nonlin;                       /* number of nonlinear constraints */

  int hasPosFreqCons;               /* any position frequency constraints? */
  int numLinFreqCons;               /* number of linear frequency constraints */
  int hasSubCon;                    /* any submotif constraints? */
  int subConIndex;                  /* index of the submotif constraints among the collection of nonlinear constraints */

  int numProfParms;                 /* number of parameters needed to parameterize the information content profile */
  int profParmIndex;                /* index of first such parameter in the collection of parameters to estimate */
  double sharpestBound;             /* max of all lower bounds on nucleotide frequencies */

  int *linConMatrix;                /* matrix encoding all linear constraints */
  double *low;                      /* lower bounds on parameters to be estimated */
  double *up;                       /* upper bounds on parameters to be estimated */
  NLCON *nlCons;                    /* pointer to first nonlinear constraint */
} MODEL;

/* a posterior probability for the occurrence of a motif in a given position of a particular sequence */
typedef struct postProb{
  int seqNum;                       /* the number of the sequence for which we calculate the posterior probablitiy */
  int start;                        /* the position in that sequence */

  double prob;                      /* the posterior probability */
  BOOLEAN revComp;                  /* is the motif on the reverse complement? */
} POSTPROB;


/* probability for a motif log-transformed into an integer value */
typedef struct intMotProb{
  int *seq;                         /* pointer to the nucleotide at which the motif starts */
  int prob;                         /* log-transformed probability as integer */
} INTMOTPROB;


/* one sequence in the dataset */
typedef struct sample{

  char *name;		            /* the name of the sequence */
  long nameLength;                  /* the length of the name */

  int *seq;		            /* integer-encoded sequence with 0=A, 1=C, 2=G, 3=T */
  int *rcSeq;                       /* reverse complement of the sequence, 5' to 3' as well */
  long length;	                    /* length of the sequence */

  int cvGroup;                      /* validation group that this sequence currently belongs to */
  BOOLEAN exclude;                  /* exclude this sequence during current fitting (for CV)? */
  int *numCuts;                     /* number of cuts this sequence has been subdivided into */

  double *logCumBack;               /* cumulative log probabilities for sequence under the background model */
  double *backLR;                   /* LR contribution of the background given that the motif starts in a given position */
  double *cumProbBack;              /* cumulative background probabilities for different motif widths, needed for TCM likelihood */


  double *mProbs1;                  /* probability of motif under given PWM at each starting position */
  double *rcmProbs1;                /* probability of motif under given PWM at each starting position of reverse complement */

  double *mProbs2;                  /* probability of motif under given PWM at each starting position */
  double *rcmProbs2;                /* probability of motif under given PWM at each starting position of reverse complement */

  double *seqProbs;                 /* probability contribution of a motif starting in a given position of forward strand */    
  double *rcSeqProbs;               /* probability contribution of a motif starting in a given position of reverse strand */    
  
  double seqLR;                     /* likelihood ratio */
  double *cutLRs;                   /* likelihood ratios for cuts */

  int dum;                          /* dummy variable for padding to avoid odd segmentation faults under Linux64*/

} SAMPLE;


/* a parameter estimate */
typedef struct theta{
  
  MTYPE mType;                       /* model type under which parameters were estimated*/

  int width;                         /* width of motif */
  double *pwm;                       /* position weight matrix */

  double intensity;                  /* intensity parameter:
                                        for ZOOPS, CUT, and OVER models, this is pi
					for TCM model, this is lambda */
  int numSites;                      /* corresponding number of expected motif occurrences */

  int numProfParms;                  /* number of profile shape parameters */
  double *profParms;                 /* profile shape parameters */

  BOOLEAN converged;                 /* did the algorithm converge for this parameter estimate? */
  double logLR;                      /* the log likelihood ratio for this parameter estimate wrt to the background model */
  double logLik;                     /* the log likelihood for this parameter estimate */

  double KL;                         /* the Kullback-Leibler divergence for this parameter estimate */
  double *posKL;                     /* array of KL-divergences for each of the motif positions */

  double IC;                         /* average information content of the estimated PWM */
  double *posIC;                     /* array of information contens in each of the positions of the PWM */

  double logEval;                    /* the log E-value for this parameter estimate */
  double *colScores;                 /* array of column scores to calculate the E-value */

  double aic;                        /* AIC for this parameter estimate */
  double bic;                        /* BIC for this parameter estimate */


  double score;                      /* score (if this is a starting value) */
 
  int conSetNum;                     /* number of the constraint set under which parameters were estimated */
  MODEL *conMod;                     /* interpreted constraint set under which parameters were estimated */
} THETA;


/* starting values found based on the E-value criterion */
typedef struct startSpace{
  MTYPE mType;                       /* model type for which starting values were found */

  int nnSites;                       /* number of candidate values for the expected number of motif occurrences */
  int *numSites;                     /* array of these candidate values */

  int numSv;                         /* number of starting values for each such candidate value */
  int totSv;                         /* total number of starting values that are allocate in svTheta */
  double *svLogLR;                   /* array of log LR for the candidate starting values (for likelihood-based starting values) */

  THETA *svTheta;                    /* array of starting values */
} SVSPACE;



/* choice for model parameters */
typedef struct parmChoice {

  MTYPE mType;                       /* model type */ 
  int modTypeNum;                    /* number of model type */
  int width;                         /* width of motif */
  int cSetNum;                       /* constraint set number */
  int order;                         /* order of background model */

} PCHOICE;

/* the dataset */
typedef struct dataSet{

  /* Input sequences */
  char *seqFileName;                 /* name of the sequence file */    
  FILE *seqFile;                     /* sequence file */
  SAMPLE *samples;                   /* array of samples */
  int numSeqs;                       /* number of sequences */
  int maxSeqLength;                  /* maximum sequence length */
  int *len;                          /* space for sorting sequence lengths */
  long totalNucs;                    /* total number of nucleotides */
  double *nucFreq;                   /* nucleotide frequencies in supplied sequences*/
  int maxNameLength;                 /* maximum length sequence names */

  /* Basic model parameters */
  int aLength;                       /* alphabet length */
  BOOLEAN revComp;                   /* consider motif occurrences on reverse complement? */
  BOOLEAN R;                         /* is cosmo being run in R?*/

  int minWidth;                      /* lower bound on the motif widths to consider */
  int maxWidth;                      /* upper bound on the motif widths to consider */
  int numWidths;                     /* number of candidate widths to consider altogether */

  int minNumSites;                   /* minimum number of expected motif occurrences in all sequences */
  int zoopsMaxNumSites;              /* maximum number of expected motif occurrences in all sequences for ZOOPS model*/
  int tcmMaxNumSites;                /* maximum number of expected motif occurrences in all sequences for TCM model*/  
  BOOLEAN findBestNumSites;          /* find the best number of sites for an estimated PWM in fitModel()? */
  BOOLEAN maxIntensity;              /* maximize intensity parameter during constrained maximization of the likelihood? */

  DISTR *distrs;                     /* array of distributions for log-likelihood ratios for number of sites N=1,...,maxSites */
  int ndistrs;                       /* largest N for which distribution is known */

  /* The candidate model types */
  MTYPE *mTypes;                     /* model types to try: OOPS, ZOOPS, TCM */
  int numModTypes;                   /* number of model types to consider */
  BOOLEAN oops;                      /* consider OOPS model? */
  BOOLEAN zoops;                     /* consider ZOOPS model? */
  BOOLEAN tcm;                       /* consider full TCM model? */
  MTYPE approx;                      /* approximate model type for TCM model */
  char *modTypeName;                 /* string to hold a particular model type name */
  int cutFactor;                     /* cuts are of length (cutFactor)x(motif width)*/
  int *numCuts;                      /* array of number of cuts that dataset was divided into, one for each candidate width */
  int *cutLengths;                   /* array of cut lengths, one for each candidate width*/

  /* The constraints */
  char *conFileName;                 /* name of the file containing the constraints */ 
  FILE *conFile;                     /* constraint file */
  CONSTRING conString;               /* constraint file read into CONSTRING struct */
  int numConSets;                    /* number of constraint sets to consider*/
  CONSET *cSets;                     /* array of constraint sets */
  BOOLEAN hasConstraints;            /* are there any constraints? */
  BOOLEAN addFree;                   /* add unconstrained case to list of given constraint sets? */
  int maxNumProfParms;               /* maximum number of parameters needed to parameterize information content profile */
  int maxNumInts;                    /* maximum number of intervals over all constraint sets */
  MODEL *mod;                        /* array of interpreted constraint sets */
  int numModels;                     /* number of such interpreted constraint sets */
  int effNumModels;                  /* number of models that will actually be considered (possibly excluding empty constraint sets)*/
  int *intNumBasePairs;              /* number of base pairs in each interval */
  int *intFirst;                     /* first motif position in each interval */
  int *intLast;                      /* last position in each interval */
  char *line;                        /* a line in reading in constraint file */
  char *word;                        /* a word in reading in constraint file */
  char *first;                       /* first token in reading in constraint file */
  long maxN;                         /* maximum number of parameters */
  long maxNlin;                      /* maximum number of linear parameters */
  long maxNonlin;                    /* maximum number of non-linear parameters */

  /* The background model */
  BOOLEAN hasBackFile;               /* is there a separate file of sequences from which to estimate background model? */
  char *backFileName;                /* name of the sequence file to estimate background model from */
  FILE *backFile;                    /* file containing background sequences */     
  SAMPLE *backSamples;               /* array of samples to estimate background model from */
  int numBackSeqs;                   /* number of sequences to estimate background model from */ 
  long maxBackSeqLength;             /* maximum length of background sequences */
  double *backProb;                  /* space to hold 4 letter probabilies given the previous k nucleotides */

  BOOLEAN orderByCV;                 /* choose order of background Markov model by CV? */
  int order;                         /* order of background Markov model */
  int backFold;                      /* fold for likleihood-based CV for choosing order */
  int maxOrder;                      /* maximum order for background Markov model to consider */
  double *backKL;                    /* KL divergence for the various candidate orders */
  char *markovDesc;                  /* string describing the choice of the order for the background Markov model */

  BOOLEAN readTransMat;              /* read in transition matrix for background model that is supplied by user? */
  char *transMatFileName;            /* name of the file containing the transition matrix for the background Markov model */
  FILE *transMatFile;                /* transition matrix file */

  BOOLEAN readBfile;                 /* read in MEME-style background file? */
  char *bfileName;                   /* name of MEME-style background file? */
  FILE *bfile;                       /* MEME-style background file */
  char **fields;                     /* array of two strings for holding strings read in from MEME-styl bfile */
  double **transMats;                /* transition matrices for background Markov model */

  
  /* Additional output */
  BOOLEAN xml;                       /* generate XML output rather than text output? */
  BOOLEAN printStatus;               /* print status for web application */

  
  BOOLEAN output4sim;                /* generate output for simulation purposes? */
  char *simFileName;                 /* name of file to which simulation output goes */
  FILE *simFile;                     /* file containing simulation output */
  double elapsedTime;                /* runtime of program */


  BOOLEAN outputProbs;               /* outpur estimated posterior probabilities? */
  char *probFileName;                /* name of the output file for posterior probabilities */
  FILE *probFile;                    /* posterior probability output file */


  BOOLEAN outputROC;                 /* generate output for calculation of ROC curves? */
  char *rocFileName;                 /* name of file to which ROC output goes */
  FILE *rocFile;                     /* file containing ROC output */
  

  /* Variables for finding starting values */
  int numStarts;                     /* number of starting values to consider for each intensity value */
  BOOLEAN likStarts;                 /* use likelihood-based starting values rather than E-value based starting values? */
  double hiProb;                     /* for finding starting values, probability to assign to nucleotide actually observed in oligo */
  double lowProb;                    /* for finding starting values, probability to assign to other nucleotides */

  BOOLEAN cvStarts;                  /* find starting values based on the training set only for pwmCV? */
  SVSPACE *svSpace;                  /* space for starting values */
  SVSPACE *cvSvSpace1;               /* 1st space for starting values for CV */
  SVSPACE *cvSvSpace2;               /* 2nd space for starting values for CV */
  INTMOTPROB *intMotProbs1;          /* integer-valued motif probabilities */
  INTMOTPROB *intMotProbs2;          /* another space for integer-valued motif probabilities */
  INTMOTPROB *maxMotProbs;           /* sorted integer-valued motif probabilities */


  /* Variables needed for cross-validation */
  double *logLiks;                   /* space for log likelihoods of all sequences (for finding truncation points in cv)*/
  double klQuant;                    /* specified quantile of Kullback-Leibler divergences */
  int fold;                          /* length of that array = current fold */
  THETA *refThetas;                  /* array of unconstrained Thetas */
  BOOLEAN *refThetaConv;             /* array of indicator showing whether a given refTheta did converge */
  int nRefThetas;                    /* number of reference thetas */
  
  /* Various small scratch spaces */
  double *pwm1;                      /* space for a position weight matrix */
  double *pwm2;                      /* another space for a position weight matrix */
  int *lPwm1;                        /* space for a position weight matrix that has been log-transformed in integer format */
  int *lPwm2;                        /* space for another position weight matrix that has been log-transformed in integer format */
  double *posGrad;                   /* space for gradient wrt to position weight matrix parameters */
  double *cumProb;                   /* array of cumulative background probabilities */
  double *motifCounts;               /* space for observed nucleotide counts for a particular alignment */
  THETA *svTheta;                    /* a starting value for theta */
  THETA *theta;                      /* the final parameter estimates */
  THETA *theta1;                     /* space to create a copy of theta */
  THETA *theta2;                     /* space to create a copy of theta */

  int *IP;                           /* I'_i[n] */
  int *minI;                         /* minimum intermediate value of I */
  int *maxI;                         /* maximum intermediate value of I */
  double *logP;                      /* log N! */
  double *logSP;		     /* log script_P[# samples][LLR] */
  long Irange;                       /* maxI-minI+1 */

  /* Choice of model fine-tuning parameters */
  CRIT crits;                        /* final choice of model parameters */
  CRITDEF intCrit;                   /* criterion for finding intensity parameter */
  CRITDEF wCrit;                     /* criterion for choosing width */
  char *wDesc;                       /* string describing the choice of the motif width */
  CRITDEF modCrit;                   /* criterion for choosing model type */
  char *modDesc;                     /* string describing the choice of the model type */
  CRITDEF conCrit;                   /* criterion for choosing constraint set */
  char *conDesc;                     /* string describing the choice of the constraint set */
  CRIT *allCrits;                    /* summary of all considered models */

  /* optimization parameter */
  int fxUpdated;                     /* has fx already been called to evaluate seqLLR? */

  /* Posterior probabilities */
  POSTPROB *postProbs;               /* posterior starting probabilities */
  POSTPROB *maxPostProbs;            /* sorted posterior probabilities */

  /* cosmo version */
  char *version;                     /* version of cosmo */
  char *command;                     /* command with which cosmo was called */
  char *reference;                   /* reference article */
  char *donlpReference;              /* reference for donlp */
} DATASET;


  
  


