/************************************************************************
*								       	*
*	MEME							       	*
*	Author: Timothy L. Bailey				       	*
*									*
*	Copyright							*
*	(1994 - 2000) The Regents of the University of California.	*
*	All Rights Reserved.						*
*									*
*	Permission to use, copy, modify, and distribute any part of 	*
*	this software for educational, research and non-profit purposes,*
*	without fee, and without a written agreement is hereby granted, *
*	provided that the above copyright notice, this paragraph and 	*
*	the following three paragraphs appear in all copies.		*
*									*
*	Those desiring to incorporate this software into commercial 	*
*	products or use for commercial purposes should contact the 	*
*	Technology Transfer Office, University of California, San Diego,*
*	9500 Gilman Drive, La Jolla, California, 92093-0910, 		*
*	Ph: (619) 534 5815.						*
*									*
*	IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO 	*
*	ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR 	*
*	CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF 	*
*	THE USE OF THIS SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA 	*
*	HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 		*
*									*
*	THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE *
*	UNIVERSITY OF CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE 		*
*	MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.  *
*	THE UNIVERSITY OF CALIFORNIA MAKES NO REPRESENTATIONS AND 	*
*	EXTENDS NO WARRANTIES OF ANY KIND, EITHER EXPRESSED OR IMPLIED, *
*	INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 	*
*	MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, OR THAT 	*
*	THE USE OF THE MATERIAL WILL NOT INFRINGE ANY PATENT, 		*
*	TRADEMARK OR OTHER RIGHTS.  					*
*								       	*
************************************************************************/

#include <math.h>
#include <limits.h>

#define RNDEPS 1e-12
#define MSN 24          /* maximum length of sequence name */

#define SMALL INT_MIN

typedef void *malloc_t;
 
/* the largest and smallest double precision numbers */
#define BIG DBL_MAX
#define LITTLE -BIG

/* 
  rounding 
*/
/* macro to round to the nearest int value, except halfway cases are rounded
   to the int value larger in magnitude.
*/

#define exp10(X) pow(10.0, (X))
#define log10(X) (log(X)/2.30258509299405)
extern double rint(); /* missing in math.h ??? */

#define NINT(x) ((int)((x) >= 0 ? ((x) + 0.5) : ((x) - 0.5)))
/* macro to round to the nearest int value, except halfway cases are rounded
   to the int value smaller in magnitude.
*/
#define _H_ 0.499999999999999
#define NINTL(x) ((int)((x) >= 0 ? ((x) + _H_) : ((x) - _H_)))
/*
  round x to d significant digits and put in y
\*/
#define RNDDIG 14
#define RND(x, d, y) {							\
  if (x > 0) {								\
    double _z_ = exp10(ceil((d)-1-log10(x)));				\
    y = rint(_z_*(x))/_z_;						\
  } else if (x < 0) {							\
    double _z_ = exp10(ceil((d)-1-log10(-x)));				\
    y = -rint(_z_*(-x))/_z_;						\
  } else {								\
    y = 0;								\
  }									\
}

/* max and min for all seasons */
#define MIN(X,Y) ((X) < (Y) ? (X) : (Y))
#define MAX(X,Y) ((X) > (Y) ? (X) : (Y))


/* Used for preventing overflow with logarithms */
#define EPSILON 1e-200		/* smallest LOG2(X) = -664.385 */ 
#define LOG(X) log((double)((X) + EPSILON))
#define DELOG(X) (exp((double)(X)) - EPSILON)
#define LOG2(X) (LOG(X)/0.6931471805599452862)
/*#define LOG2(X) log2((double)((X) + EPSILON))*/

/* used for summing logarithms:  log(x + y) where log(x) and log(y) are avail.*/
#define BITS 64.0
#define LOGZERO -1e100			/* log of number close to zero */
#define LOGZEROI -2147483647		/* log of number close  zero (integer)*/
#define LOG_SUM1(logx, logy) (((((logy)<=LOGZERO) || ((logx)-(logy))>BITS)) ? \
    (logx) : (logx) + log( 1 + exp((logy) - (logx) ) ) )
#define LOG_SUM(logx, logy) ( ( (logx) > (logy) ) ? \
  LOG_SUM1( (logx), (logy) ) : LOG_SUM1( (logy), (logx) ) )


/* Largest X  L = e^(-1/SCALE_LOGS); larger X yeild INT_LOG(x) = 0 */
#define SCALE_LOGS 1e6			/* don't use larger! */
#define INT_LOG(X) ((int)(SCALE_LOGS * LOG(X)))
#define INT_DELOG(X) DELOG((X)/SCALE_LOGS)

/* 
   get the exponent and mantissa of large numbers expressed as log10(x)
   for printing with prec digits after the decimal point in mantissa
*/
#define exp10_logx(logx, m, e, prec) { \
  (e) = floor(logx); \
  (m) = exp10((logx) - (e)); \
  if (m+(.5*pow(10,-prec)) >= 10) { (m) = 1; (e) += 1;} \
}
/*
	Malloc stuff for improved debugging 
*/


/* make a better free function */
#define myfree(P) {							\
  if (P) free((char *) (P));						\
}

/*
  Split(string, array, len)
  Split a null-terminated string on whitespace.
  Place results into a array of strings.
  len is the length of the array.
  The array is freed if it initially is non-NULL.
  Don't change the values in array or it may cause crashes!
*/

#define Split(string, array, len)                                           \
{								            \
  int i, i1, i2, l;			/* position in string */            \
                                                                            \
                                                                            \
  for (l=0, i1=0, i2=0; l<len; l++, i1=i2) {		                    \
    while (string[i1]==' ' || string[i1]=='\t' || string[i1]=='\n') i1++;   \
    if (string[i1]=='\0') break;				            \
    i2 = i1;							            \
    while (string[i2]!=' ' && string[i2]!='\t' && string[i2]!='\n' &&       \
      string[i2]!='\0') i2++;			                            \
    for (i=0; i1<i2; i++, i1++) array[l][i] = string[i1];	            \
    array[l][i] = '\0';						            \
  }								            \
}								          
