/******************************************************************************
*                                                                             *
*      cosmo - Constrained search for motifs in DNA sequences                 *
*      Author: Oliver Bembom                                                  *
*                                                                             *
*      This file is part of cosmo.                                            *
*                                                                             *
*      Conditions of use:                                                     *
*                                                                             *
*    1. cosmo is under the exclusive copyright of Oliver Bembom,              *
*       (bembom@berkeley.edu), the Regents of the University of California,   *
*       and Peter Spellucci (spellucci@mathematik.tu-darmstadt.de).           *
*                                                                             *
*    2. Permission to use, copy, modify, and distribute any part of this      *
*       software for research purposes, without fee and without a written     *
*       agreement, is hereby granted, provided that this license appears in   *
*       all copies. Commercial uses require permission and licensing from     *
*       Oliver Bembom, the Regents of the University of California, and       *
*       Peter Spellucci.                                                      *
*                                                                             *
*    3. If you use this software in your research, please cite                *
*                                                                             *
*       O. Bembom, S. Keles, M.J. van der Laan (2007). Supervised Detection   *
*       of Conserved Motifs in DNA Sequences with cosmo. Statistical          *
*       Applications in Genetics and Molecular Biology: Vol. 6 : Iss. 1,      *
*       Article 8. http://www.bepress.com/sagmb/vol6/iss1/art8.               *
*                                                                             *
*       cosmo makes use of the donlp2() function by Peter Spellucci.          *
*       The use of donlp2() must be acknowledged in any publication which     *
*       contains results obtained with cosmo or parts of it. Citation of the  *
*       author's name and netlib-source is suitable.                          *
*                                                                             *
*    4. In no event shall the author be liable to any party for direct, in-   *
*       direct, special, incidental, or consequential damages, including lost *
*       profits, arising out of the use of this software, even if the author  *
*       has been advised of the possibility of such damage.                   *
*                                                                             *
*    5. cosmo is provided on an 'as is' basis, and the author has no obli-    *
*       gations to provide maintenance, support, updates, enhancements, or    *
*       modifications. The author makes no representations and extends no     *
*       warranties of any kind, either expressed or implied, including, but   *
*       not limited to, the implied warranties of merchantability or fitness  *
*       for a particular purpose, or that the use of the material will not    *
*       infringe any patent, trademark or other rights.                       *
*                                                                             *
*                                                                             *
******************************************************************************/

#include <Rdefines.h>
#include <R_ext/Rdynload.h>

void ccosmo(
	   int *seqs,
	   int *numSeqs,
	   int *maxSeqLength,
	   char **constraints,
	   int *minWidth,
	   int *maxWidth,
	   int *revComp,
	   int *minSites,
	   int *maxSites,
	   int *oops,
	   int *zoops,
	   int *tcm,
	   char **approx,
	   int *cutFac,
	   char **wCrit,
	   int *wFold,
	   int *wTrunc,
	   char **modCrit,
	   int *modFold,
	   int *modTrunc,
	   char **conCrit,
	   int *conFold,
	   int *conTrunc,
	   char **intCrit,
	   int *intFold,
	   int *intTrunc,
	   int *maxIntensity,
	   int *starts,
	   int *lstarts,
	   int *backSeqs,
	   int *numBackSeqs,
	   int *maxBackSeqLength,
	   int *backFold,
	   double *transMat,
	   int *order,
	   int *maxOrder,
	   int *selOrder,
	   double *klDivs,
	   double *estTransMat,
	   double *selIntCrit,
	   double *wCrits,
	   double *modCrits,
	   double *conCrits,
	   int *wInit,
	   int *modInit,
	   int *conInit,
	   int *selWidth,	   
	   int *selModel,
	   int *selConSet,
	   int *selNumSites,
	   int *numModels,
	   double *pwm,
	   double *motProbs,
	   int *motRevComp,
	   int *alignSeqs,
	   int *alignStarts,
	   double *eVal,
	   int *success,
	   int *msgLevel
	   );

void bgm(
	 int *backSeqs,
	 int *numBackSeqs,
	 int *maxBackSeqLength,
	 int *backFold,
	 int *order,
	 int *maxOrder, 
	 double *estTransMat,
	 double *klDivs,
	 int *selOrder,
	 int *success
	 );


static R_NativePrimitiveArgType ccosmo_t[59]= {INTSXP,INTSXP,INTSXP,STRSXP,INTSXP,INTSXP,INTSXP,INTSXP,INTSXP,INTSXP,INTSXP,INTSXP,STRSXP,INTSXP,STRSXP,INTSXP,INTSXP,STRSXP,INTSXP,INTSXP,STRSXP,INTSXP,INTSXP,STRSXP,INTSXP,INTSXP,INTSXP,INTSXP,INTSXP,INTSXP,INTSXP,INTSXP,INTSXP,REALSXP,INTSXP,INTSXP,INTSXP,REALSXP,REALSXP,REALSXP,REALSXP,REALSXP,REALSXP,INTSXP,INTSXP,INTSXP,INTSXP,INTSXP,INTSXP,INTSXP,INTSXP,REALSXP,REALSXP,INTSXP,INTSXP,INTSXP,REALSXP,INTSXP,INTSXP};

static R_NativePrimitiveArgType bgm_t[10]= {INTSXP,INTSXP,INTSXP,INTSXP,INTSXP,INTSXP,REALSXP,REALSXP,INTSXP,INTSXP};

static const  R_CMethodDef cMethods[] = {
      {"ccosmo", (DL_FUNC) &ccosmo, 59,ccosmo_t},
      {"bgm", (DL_FUNC) &bgm, 10,bgm_t},
      {NULL, NULL, 0}
    };

/*
 JHB we may need to change this for windows.
*/  
void R_init_cosmo(DllInfo *info)
{
  R_registerRoutines(info,cMethods,NULL,NULL,NULL);
}
